from sphinx.util.docutils import SphinxDirective
from docutils import nodes
from docutils.parsers.rst import directives
from sphinx.writers.html import HTMLTranslator

from .common import (
    BaseNode,
    add_role_and_directive,
    add_visit_depart,
    GenericDirective,
    BaseClassNode,
)

TAG = 'video'
VIDEO_TARGET_ATTRIBUTE_NAME = 'poster'
SOURCE_TARGET_ATTRIBUTE_NAME = 'src'

VIDEO_ATTRIBUTES = [
    'autoplay',
    'muted',
    'loop',
]

class video_node(BaseClassNode):
    def __init__(self, *args, dark: bool = False, **kwargs):
        super().__init__(*args, **kwargs)
        self.add_class('background-video')
        if dark:
            self.add_class('dark')
        self.add_data(**kwargs)
        self.add_attributes(
            {attr: None for attr in VIDEO_ATTRIBUTES}
        )

    def add_data(self, poster_target: str = None, video_target: str = None):
        if poster_target is not None:
            self.add_target('poster', poster_target)
        if video_target is not None:
            source = video_source_node(target=video_target)
            self += source
    
    def add_target(self, attribute_name: str, target: str):
        a = 'attributes'
        if a not in self:
            self[a] = {}        
        self[a].update({
            attribute_name: target,
        })


class video_source_node(BaseClassNode): 
    def __init__(self, *args, target: str = None, **kwargs):
        super().__init__(*args, **kwargs)
        if target is not None:
            self.add_attributes({
                'src': target,
            })
        # TODO: LIkely need to set this based on target
        self.add_attributes({
            'type': "video/mp4",
        })

class VideoDirective(GenericDirective):
    node_type = video_node
    required_arguments = 2

    def super(self):
        node = super().run()[0]
        node.add_data(
            poster_target=self.arguments[0],
            video_target=self.arguments[1],
        )
        return [node]

def setup_video(app):
    app.add_node(video_node)
    app.add_node(video_source_node)
    app.add_directive('video', VideoDirective)

class VideoTranslator(HTMLTranslator):
    pass
add_visit_depart(VideoTranslator, video_node.__name__, TAG)
add_visit_depart(VideoTranslator, video_source_node.__name__, 'source')




from copy import deepcopy

from sphinx.util.docutils import SphinxDirective
from docutils import nodes
from docutils.parsers.rst import directives
from sphinx.writers.html import HTMLTranslator

from .div import div_wrap_node
from .span import span_node
from .common import (
    BaseNode,
    add_role_and_directive,
    add_visit_depart,
    GenericDirective,
)

TAG = 'header'

class header_node(BaseNode): pass
class footer_node(BaseNode): pass

def make_header_node():
    return make_node(header_node)

def make_footer_node():
    return make_node(footer_node)

def make_node(node_type):
    node = node_type()
    div = div_wrap_node()
    node += div
    active_node = div
    return node, active_node

def set_alignment(options: dict, active_node):
    ha = 'horizontal-alignment'
    if ha in options:
        requested_alignment = options[ha].lower()
        good_alignment = False
        # TODO: can it do center?
        if requested_alignment == 'right':
            class_alignment = 'alignright'
            good_alignment = True
        elif requested_alignment == 'left':
            class_alignment = 'alignleft'
            good_alignment = True
        elif requested_alignment == 'center':
            class_alignment = 'aligncenter'
            good_alignment = True
        else:
            print(f'ERROR: Bad requested header alignment {requested_alignment}, ignoring...')
        if good_alignment:
            span = span_node()
            span.add_class(class_alignment)
            active_node += span
            active_node = span
        del(options[ha])
    return active_node

class HeaderDirective(GenericDirective):
    node_type = header_node

    def run(self):
        node, active_node = make_node(self.node_type)        
        active_node = set_alignment(self.options, active_node)
        self._process_content(active_node)
        return [node]

class FooterDirective(HeaderDirective):
    node_type = footer_node

class HeaderConfigDirective(GenericDirective):
    node_type = header_node
    config_name = 'slide_header'
    option_spec = deepcopy(GenericDirective.option_spec)
    option_spec.update({
        'no-wrap': directives.unchanged,
    })

    def run(self):
        if 'no-wrap' in self.options:
            node = self.node_type()
            active_node = node
        else:
            node, active_node = make_node(self.node_type)        
        self.set_style(node)
        self._process_content(active_node)
        self.env.app.config[self.config_name] = deepcopy(node)
        return []

class FooterConfigDirective(HeaderConfigDirective):
    node_type = footer_node
    config_name = 'slide_footer'

def header_footer_default(app, docname, source):
    for item in ['slide_header', 'slide_footer']:
        try:
            del(app.config[item])
        except AttributeError:
            pass # config value not present, OK

def setup_header_footer(app):
    app.add_node(header_node)
    app.add_node(footer_node)
    app.add_directive('header-config', HeaderConfigDirective)
    app.add_directive('header', HeaderDirective)
    app.add_directive('footer-config', FooterConfigDirective)
    app.add_directive('footer', FooterDirective)
    app.connect('source-read', header_footer_default)

class HeaderFooterTranslator(HTMLTranslator):
    pass

add_visit_depart(HeaderFooterTranslator, header_node.__name__, TAG)
add_visit_depart(HeaderFooterTranslator, footer_node.__name__, 'footer')


from sphinx.util.docutils import SphinxDirective
from docutils import nodes
from docutils.parsers.rst import directives
from sphinx.writers.html import HTMLTranslator

from .common import (
    BaseNode,
    add_role_and_directive,
    add_visit_depart,
)

TAG = 'pre'

class preformatted_node(BaseNode): pass

def setup_preformatted(app):
    app.add_node(preformatted_node)
    add_role_and_directive(app, preformatted_node, 'preformatted')
    add_role_and_directive(app, preformatted_node, 'pre')

class PreformattedTranslator(HTMLTranslator):
    pass

add_visit_depart(PreformattedTranslator, preformatted_node.__name__, TAG)


from copy import deepcopy

from sphinx.util.docutils import SphinxDirective
from docutils import nodes
from docutils.parsers.rst import directives
from sphinx.writers.html import HTMLTranslator

from .paragraph import paragraph_node

TAG = 'blockquote'

from .common import (
    BaseClassNode,
    BaseNode,
    add_role_and_directive,
    add_visit_depart,
    GenericDirective,
    create_role,
)

class blockquote_node(BaseNode): pass
class text_quote_node(BaseClassNode):
    classes = ['text-quote']
class cite_node(BaseNode): pass

class TextQuoteDirective(GenericDirective):
    node_type = text_quote_node
    option_spec = deepcopy(GenericDirective.option_spec)
    option_spec.update({
        'citation': directives.unchanged
    })

    def run(self):
        node = super().run()[0]
        cite = 'citation'
        if cite in self.options:
            citation = cite_node()
            paragraph = paragraph_node()
            citation += nodes.Text(self.options[cite])
            paragraph += citation
            node += paragraph
        return [node]

class BlockquoteDirective(TextQuoteDirective):
    node_type = blockquote_node

def setup_blockquote(app):
    app.add_node(text_quote_node)
    app.add_node(cite_node)
    app.add_role('text-quote', create_role(text_quote_node))
    app.add_role('blockquote', create_role(blockquote_node))
    app.add_directive('text-quote', TextQuoteDirective)
    app.add_directive('blockquote', BlockquoteDirective)
    add_role_and_directive(app, cite_node, 'cite')

    # Convenience
    app.add_role('tq', create_role(text_quote_node))
    app.add_role('bq', create_role(blockquote_node))

class BlockquoteTranslator(HTMLTranslator):
    pass

add_visit_depart(BlockquoteTranslator, text_quote_node.__name__, TAG)
add_visit_depart(BlockquoteTranslator, blockquote_node.__name__, TAG)
add_visit_depart(BlockquoteTranslator, cite_node.__name__, 'cite')


from codecs import open
from os import path
from setuptools import setup, find_packages
from subprocess import check_output

here = path.abspath(path.dirname(__file__))

# check_output(
#     'pandoc --from=markdown --to=rst --output=' + path.join(here, 'README.rst') + ' ' + path.join(here, 'README.md'),
#     shell=True
# )

# with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
#     long_description = f.read()
with open ('README.md', 'r') as fh:
    long_description = fh.read()

install_requires = list()
with open(path.join(here, 'requirements.txt'), 'r', encoding='utf-8') as f:
    for line in f.readlines():
        install_requires.append(line)

setup(
    name='sphinx-webslides-builder',

    version='0.0.13',

    description='sphinx builder that outputs webslides',

    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://github.com/creativerigor/webslides-extension',

    author='Creative Rigor, LLC',

    author_email='creativerigor@gmail.com',

    license='MIT',

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
    ],

    keywords='sphinx docs documentation webslides',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=install_requires,
    
    package_data={'sphinx_webslides_builder/themes/webslides_base': [
        'theme.conf',
        '*.html',
        'static/css/*.css',
        'static/js/*.js',
        'static/css/*.woff',
        'static/css/*.woff2',
    ]},

    include_package_data=True,

    entry_points = {
        'sphinx.builders': [
            'webslides = sphinx_webslides_builder',
        ],
    }
)
